@@ -43,10 +43,10 @@
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #else
-#define VERSION "2.2.3"
+#define VERSION "2.2.5"
 #define PACKAGE_NAME "mp3splt"
 #endif
-#define MP3SPLT_DATE "18/01/09"
+#define MP3SPLT_DATE "16/05/09"
 #define MP3SPLT_AUTHOR1 "Matteo Trotta"
 #define MP3SPLT_AUTHOR2 "Alexandru Munteanu"
 #define MP3SPLT_EMAIL1 "<mtrotta AT users.sourceforge.net>"
@@ -73,7 +73,12 @@ typedef struct {
   short o_option; short d_option; short k_option;
   //custom tags, no tags, quiet option
   short g_option; short n_option; short q_option;
+  short x_option;
   short N_option;
+  short O_option;
+  short T_option;
+  //export cue
+  //short E_option;
   //-Q option
   short qq_option;
   //info -i option, m3u file option
@@ -331,6 +336,8 @@ void show_small_help_exit(main_data *data)
       " -v   Prints current version and exits\n"
       " -h   Shows this help\n"
       "\n(other options)\n"
+      /*" -P   Pretend to split, without actually splitting the file\n"
+      " -E   Export .cue file (use with -P to export .cue file without splitting)\n"*/
       " -1   For mp3 files, force output tags as version 1\n"
       " -2   For mp3 files, force output tags as version 2\n"
       "      (default is to set the same version as the file to split)\n"
@@ -340,8 +347,9 @@ void show_small_help_exit(main_data *data)
       " -p + PARAMETERS (th, nt, off, min, rm, gap): user arguments for -s and -a.\n"
       " -o + FORMAT: output filename pattern. Can contain those variables:\n"
       "      @a: artist tag, @p: performer tag (might not exists), @b: album tag\n"
-      "      @t: title tag, @n: track number tag (a digit may follow the 'n' for\n"
-      "      the number of digits to output), @f: original filename\n"
+      "      @t: title tag, @n: track number identifier, @N: track tag number\n"
+      "      (a digit may follow the 'n' or 'N' for the number of digits to output),\n"
+      "      @f: original filename\n"
       " -g + TAGS : custom tags for the split files.\n"
       "      TAGS can contain those variables : \n"
       "         @a, @b, @t, @y, @c, @n, @o (set original tags),\n"
@@ -350,12 +358,15 @@ void show_small_help_exit(main_data *data)
       "       (\% means that we set the tags for all remaining files)\n"
       " -d + DIRNAME: to put all output files in the directory DIRNAME.\n"
       " -k   Consider input not seekable (slower). Default when input is STDIN (-).\n"
+      " -O + TIME : Overlap split files with TIME (slower).\n"
       " -n   No Tag: does not write ID3v1 or vorbis comment. If you need clean files.\n"
+      " -x   No Xing header: does not write the Xing header. Use with -n if you wish\n"
+      "      to concatenate the split files\n"
       " -N   Don't create the 'mp3splt.log' log file when using '-s'.\n"
-      " -q   Quiet mode: try not prompt (if possible) and print less messages.\n"
+      " -q   Quiet mode: try not to prompt (if possible) and print less messages.\n"
       " -Q   Very quiet mode: don't print anything to stdout and no progress bar\n"
       "       (also enables -q).\n"
-      " -D   Debug mode: used to debug the program (by developers).\n\n"
+      " -D   Debug mode: used to debug the program.\n\n"
       "      Read man page for complete documentation.\n");
 
   if (console_out == stderr)
@@ -521,9 +532,10 @@ void check_args(int argc, main_data *data)
           opt->e_option || opt->i_option ||
           opt->f_option || opt->a_option ||
           opt->p_option || opt->o_option ||
-          opt->g_option || opt->n_option)
+          opt->g_option || opt->n_option ||
+          opt->x_option)
       {
-        print_error_exit("the -w option can only be used with -m, -d and -q", data);
+        print_error_exit("the -w option can only be used with -m, -d, -q and -Q", data);
       }
     }
 
@@ -536,7 +548,8 @@ void check_args(int argc, main_data *data)
           opt->f_option || opt->a_option ||
           opt->p_option || opt->o_option ||
           opt->g_option || opt->d_option ||
-          opt->n_option)
+          opt->n_option || opt->qq_option ||
+          opt->x_option)
       {
         print_error_exit("the -l option can only be used with -q", data);
       }
@@ -550,7 +563,7 @@ void check_args(int argc, main_data *data)
           opt->a_option || opt->p_option ||
           opt->g_option || opt->n_option)
       {
-        print_error_exit("the -e option can only be used with -m, -f, -o, -d, -q", data);
+        print_error_exit("the -e option can only be used with -m, -f, -o, -d, -q, -Q", data);
       }
     }
 
@@ -696,6 +709,20 @@ void check_args(int argc, main_data *data)
             " silence detection (-s option)", data);
       }
     }
+
+    if (opt->O_option)
+    {
+      if (opt->w_option || opt->e_option ||
+          opt->l_option || opt->i_option)
+      {
+        print_error_exit("the -O option cannot be used with"
+            " -w, -e, -l or -i", data);
+      }
+    }
+
+    if (opt->x_option)
+    {
+    }
   }
 }
 
@@ -1434,14 +1461,18 @@ options *new_options(main_data *data)
   opt->d_option = SPLT_FALSE; opt->k_option = SPLT_FALSE;
   opt->g_option = SPLT_FALSE; opt->n_option = SPLT_FALSE;
   opt->q_option = SPLT_FALSE; opt->i_option = SPLT_FALSE;
-  opt->N_option = SPLT_FALSE;
+  opt->N_option = SPLT_FALSE; opt->O_option = SPLT_FALSE;
+  opt->x_option = SPLT_FALSE;
+  opt->T_option = SPLT_FALSE;
   opt->qq_option = SPLT_FALSE;
   opt->m_option = SPLT_FALSE;
   opt->cddb_arg = NULL; opt->dir_arg = NULL;
-  opt->param_args = NULL; opt->custom_tags = NULL;
+  opt->param_args = NULL;
   opt->m3u_arg = NULL;
   opt->output_format = NULL;
 
+  opt->custom_tags = strdup("%[@o,@N=1]");
+
   //we put the default values for freedb search
   //by default, CDDB_CGI (cddb.cgi) port 80 on freedb2.org
   opt->freedb_search_type = SPLT_FREEDB_SEARCH_TYPE_CDDB_CGI;
@@ -1681,20 +1712,21 @@ int main(int argc, char **orig_argv)
   mp3splt_set_silence_level_function(state, get_silence_level, data->sl);
   //callback for the split files
   mp3splt_set_split_filename_function(state, put_split_file);
-  //callback for the progress bar
-  mp3splt_set_progress_function(state, put_progress_bar);
 
   //default we write mins_secs_hundr for normal split
   mp3splt_set_int_option(state, SPLT_OPT_OUTPUT_FILENAMES, SPLT_OUTPUT_DEFAULT);
-  mp3splt_set_int_option(state, SPLT_OPT_TAGS, SPLT_TAGS_ORIGINAL_FILE);
+  mp3splt_set_int_option(state, SPLT_OPT_TAGS, SPLT_CURRENT_TAGS);
 
   //parse command line options
   int option;
   //I have erased the "-i" option
-  while ((option = getopt(data->argc, data->argv, "m:SDvifkwleqnasc:d:o:t:p:g:hQN12")) != -1)
+  while ((option = getopt(data->argc, data->argv, "Tm:O:SDvifkwleqnasc:d:o:t:p:g:hQN12x")) != -1)
   {
     switch (option)
     {
+      case 'x':
+        mp3splt_set_int_option(state, SPLT_OPT_XING, SPLT_FALSE);
+        break;
       case 'h':
         show_small_help_exit(data);
         break;
@@ -1729,6 +1761,7 @@ int main(int argc, char **orig_argv)
         break;
       case 'l':
         opt->l_option = SPLT_TRUE;
+        console_out = console_err;
         break;
       case 'e':
         mp3splt_set_int_option(state, SPLT_OPT_SPLIT_MODE, SPLT_OPTION_ERROR_MODE);
@@ -1790,6 +1823,14 @@ int main(int argc, char **orig_argv)
         }
         opt->o_option = SPLT_TRUE;
         break;
+      case 'O':
+        opt->O_option = SPLT_TRUE;
+        mp3splt_set_long_option(state, SPLT_OPT_OVERLAP_TIME,
+            c_hundreths(optarg));
+        break;
+      case 'T':
+        opt->T_option = SPLT_TRUE;
+        break;
       case 't':
         mp3splt_set_int_option(state, SPLT_OPT_SPLIT_MODE, SPLT_OPTION_TIME_MODE);
         float converted_time = c_hundreths(optarg);
@@ -1802,7 +1843,7 @@ int main(int argc, char **orig_argv)
         else
         {
           print_error_exit("bad time expression for the time split.\n"
-              "Must be min.sec, read man page for details.", data);
+              "\tMust be min.sec, read man page for details.", data);
         }
         opt->t_option = SPLT_TRUE;
         break;
@@ -1811,7 +1852,11 @@ int main(int argc, char **orig_argv)
         opt->param_args = strdup(optarg);
         break;
       case 'g':
-        mp3splt_set_int_option(state, SPLT_OPT_TAGS, SPLT_CURRENT_TAGS);
+        if (opt->custom_tags)
+        {
+          free(opt->custom_tags);
+          opt->custom_tags = NULL;
+        }
         opt->custom_tags = strdup(optarg);
         opt->g_option = SPLT_TRUE;
         break;
@@ -1822,6 +1867,12 @@ int main(int argc, char **orig_argv)
         console_progress = stdout;
         fclose(stdout);
         break;
+      /*case 'E':
+        opt->E_option = SPLT_TRUE;
+        break;
+      case 'P':
+        mp3splt_set_int_option(state, SPLT_OPT_PRETEND_TO_SPLIT, SPLT_TRUE);
+        break;*/
       default:
         print_error_exit("read man page for documentation or type 'mp3splt -h'.",
             data);
@@ -1829,8 +1880,14 @@ int main(int argc, char **orig_argv)
     }
   }
 
+  //callback for the progress bar
+  if (!opt->T_option)
+  {
+    mp3splt_set_progress_function(state, put_progress_bar);
+  }
+
   //if quiet, does not write authors and other
-  if (!opt->q_option)
+  if (!opt->q_option && !opt->T_option)
   {
     print_version_authors(console_err);
   }
@@ -1935,13 +1992,8 @@ int main(int argc, char **orig_argv)
   {
     //enable to create directories from the output filenames
     mp3splt_set_int_option(state, SPLT_OPT_CREATE_DIRS_FROM_FILENAMES, SPLT_TRUE);
-
-    //we set our output format
     mp3splt_set_oformat(state, opt->output_format,&output_format_error);
-    if (output_format_error != SPLT_OUTPUT_FORMAT_AMBIGUOUS)
-    {
-      process_confirmation_error(output_format_error, data);
-    }
+    process_confirmation_error(output_format_error, data);
   }
 
   if (optind > 1)
@@ -1955,6 +2007,7 @@ int main(int argc, char **orig_argv)
 
   //enable/disable logging the silence splitpoints in a file
   mp3splt_set_int_option(state, SPLT_OPT_ENABLE_SILENCE_LOG, ! opt->N_option);
+
   //silence splitpoints log filename
   if (! opt->N_option)
   {
@@ -1984,11 +2037,11 @@ int main(int argc, char **orig_argv)
   }
 
   //print 'output format ambigous if necessary
-  if ((output_format_error == SPLT_OUTPUT_FORMAT_AMBIGUOUS)
-      && (data->number_of_splitpoints > 2))
-  {
-    process_confirmation_error(output_format_error, data);
-  }
+//  if ((output_format_error == SPLT_OUTPUT_FORMAT_AMBIGUOUS)
+//      && (data->number_of_splitpoints > 2))
+//  {
+//    process_confirmation_error(output_format_error, data);
+//  }
 
   //if we have a normal split, we need to parse the splitpoints
   int normal_split = SPLT_FALSE;
@@ -2000,11 +2053,24 @@ int main(int argc, char **orig_argv)
   }
 
   int j = 0;
+
+  if (data->number_of_filenames <= 0)
+  {
+    print_error_exit("no input filename(s).", data);
+  }
+
   if (data->number_of_filenames > 1)
   {
     fprintf(console_out,"\n");
     fflush(console_out);
   }
+
+  if ((data->number_of_splitpoints > 2) &&
+      opt->output_format && (strcmp(opt->output_format, "-") == 0))
+  {
+    print_warning("multiple splitpoints with stdout !");
+  }
+
   //split all the filenames
   for (j = 0;j < data->number_of_filenames; j++)
   {
@@ -2031,13 +2097,13 @@ int main(int argc, char **orig_argv)
       //if no error when getting the wrap files
       int wrap_files_number = wrap_files->wrap_files_num;
       int i = 0;
-      fprintf(console_out,"\n");
+      fprintf(stdout,"\n");
       for (i = 0;i < wrap_files_number;i++)
       {
-        fprintf(console_out,"%s\n",wrap_files->wrap_files[i]);
+        fprintf(stdout,"%s\n",wrap_files->wrap_files[i]);
       }
-      fprintf(console_out,"\n");
-      fflush(console_out);
+      fprintf(stdout,"\n");
+      fflush(stdout);
     }
     else
     {
@@ -2114,27 +2180,43 @@ int main(int argc, char **orig_argv)
           process_confirmation_error(err, data);
         }
 
-        //if custom tags, we put the tags
-        int ambigous = mp3splt_put_tags_from_string(state, opt->custom_tags, &err);
+        int ambiguous = mp3splt_put_tags_from_string(state, opt->custom_tags, &err);
         process_confirmation_error(err, data);
-        if (ambigous)
+        if (ambiguous)
         {
-          print_warning("tags format ambigous !");
+          print_warning("tags format ambiguous !");
         }
 
+        //for cddb, filenames are already set from the library, so 
+        //set output filenames to CUSTOM
+        int saved_output_filenames = mp3splt_get_int_option(state, SPLT_OPT_OUTPUT_FILENAMES, &err);
+        if (opt->c_option && !opt->o_option)
+        {
+          mp3splt_set_int_option(state, SPLT_OPT_OUTPUT_FILENAMES, SPLT_OUTPUT_CUSTOM);
+        }
+ 
         //we do the effective split
         err = mp3splt_split(state);
         process_confirmation_error(err, data);
 
+        //for cddb, set output filenames to its old value before the split
+        if (opt->c_option && !opt->o_option)
+        {
+          mp3splt_set_int_option(state, SPLT_OPT_OUTPUT_FILENAMES, saved_output_filenames);
+        }
+
         //print the average silence level
         if (opt->s_option)
         {
           if (sl->print_silence_level)
           {
-            float average_silence_levels = sl->level_sum / (double) sl->number_of_levels;
-            char message[256] = { '\0' };
-            snprintf(message,256," Average silence level : %.2f dB", average_silence_levels);
-            print_message(message);
+            if (sl->number_of_levels != 0)
+            {
+              float average_silence_levels = sl->level_sum / (double) sl->number_of_levels;
+              char message[256] = { '\0' };
+              snprintf(message,256," Average silence level : %.2f dB", average_silence_levels);
+              print_message(message);
+            }
           }
         }
 
